/*
 * CodePress - Real Time Syntax Highlighting Editor written in JavaScript - http://codepress.org/
 * 
 * Copyright (C) 2007 Fernando M.A.d.S. <fermads@gmail.com>
 *
 * Contributors :
 *
 * 	Michael Hurni <michael.hurni@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU Lesser General Public License as published by the Free Software Foundation.
 * 
 * Read the full licence: http://www.opensource.org/licenses/lgpl-license.php
 */

CodePress = {
	scrolling : false,
	autocomplete : false,
	clipboard : false,
	initialize : function()
	{
		if (typeof(editor)=='undefined' && !arguments[0]) return;
		chars = '|32|46|62|'; // charcodes that trigger syntax highlighting
		cc = '\u2009'; // control char
		editor = document.getElementsByTagName('pre')[0];
		editor.contentEditable = 'true';
		document.getElementsByTagName('body')[0].onfocus = function() { editor.focus(); }
		document.attachEvent('onkeydown', this.metaHandler);
		document.attachEvent('onkeypress', this.keyHandler);
		window.attachEvent('onscroll', function() { if (!CodePress.scrolling) setTimeout(function(){CodePress.syntaxHighlight('scroll')},1)});
		completeChars = this.getCompleteChars();
		setTimeout(function() { window.scroll(0,0) },50); // scroll IE to top
		// if (window.clipboardData.getData('Text') !== undefined) CodePress.clipboard = true;
	},
	keyHandler : function(evt)
	{
		charCode = evt.keyCode;
		if (completeChars.indexOf('|'+String.fromCharCode(charCode)+'|')!=-1 && CodePress.autocomplete)
		{ // auto complete
			CodePress.complete(String.fromCharCode(charCode));
		}
		else if (chars.indexOf('|'+charCode+'|') != -1)
		{ // syntax highlighting
			CodePress.syntaxHighlight('generic');
		}
		else if (charCode==13)
		{
			CodePress.onEnterkey(evt);
		}
	},
	metaHandler : function(evt)
	{
		keyCode = evt.keyCode;
		if (keyCode==9 || evt.tabKey)
		{ 
			CodePress.onTabkey(evt);
		}
		else if ((keyCode==122 || keyCode==121 || keyCode==90) && evt.ctrlKey)
		{ // undo and redo
			(keyCode==121||evt.shiftKey) ? CodePress.actions.redo() : CodePress.actions.undo(); 
			evt.returnValue = false;
		}
		else if (keyCode==34 || keyCode==33)
		{ // handle page up/down for IE
			self.scrollBy(0, (keyCode==34) ? 200 : -200); 
			evt.returnValue = false;
		}
		else if (keyCode==46 || keyCode==8)
		{ // save to history when delete or backspace pressed
			CodePress.actions.history[CodePress.actions.next()] = editor.innerHTML;
		}
		else if ((evt.ctrlKey || evt.metaKey) && evt.shiftKey && keyCode!=90) 
		{ // shortcuts = ctrl||appleKey+shift+key!=z(undo) 
			CodePress.shortcuts(keyCode);
			evt.returnValue = false;
		}
		else if (keyCode==86 && evt.ctrlKey)
		{ // handle paste
			if (CodePress.clipboard)
			{
				var text = window.clipboardData.getData('Text');
				if (text)
				{
					text = text.replace(/\t/g,'\u2008');
					CodePress.insertCode(text);
					evt.returnValue = false;
				}
			}
			setTimeout(function(){CodePress.syntaxHighlight('paste');},10);
		}
		else if ((keyCode == 67 || keyCode==88) && evt.ctrlKey)
		{ // handle copy or cut
			var range = CodePress.getRange();
			parent.CodePress.doCopy(CodePress._convertCode(range.htmlText));
			if (keyCode == 88)
			{
				range.txet = '';
				setTimeout(function(){CodePress.syntaxHighlight('generic');},100);
			}
			evt.returnValue = false;
			if (keyCode == 67) return; // to avoid setting modified flag
		}
		CodePress.onModified(evt);
	},
	onTabkey : function(evt)
	{
		top.setTimeout(function(){CodePress.syntaxHighlight('generic');},100);
		CodePress.snippets(evt);
	},
	onEnterkey : function(evt)
	{
		CodePress.syntaxHighlight('generic');
	},
	onModified : function(evt)
	{
		// do nothing
	},
	findString : function()
	{
		range = self.document.body.createTextRange();
		if (range.findText(cc))
		{
			range.select();
			range.text = '';
		}
	},
	split : function(code,flag)
	{
		if (flag=='scroll')
		{
			this.scrolling = true;
			return code;
		}
		else
		{
			this.scrolling = false;
			mid = code.indexOf(cc);
			if (mid - 2000 < 0)
			{
				ini = 0;
				end = 4000;
			}
			else if (mid + 2000 > code.length)
			{
				ini = code.length - 4000;
				end = code.length;
			}
			else
			{
				ini = mid - 2000;
				end = mid + 2000;
			}
			code = code.substring(ini,end);
			return code.substring(code.indexOf('<P>'),code.lastIndexOf('</P>')+4);
		}
	},
	syntaxHighlight : function(flag)
	{
		if (flag != 'init') document.selection.createRange().text = cc;
		o = CodePress._prepareCodeForSyntax(flag);
		x = z = this.split(o,flag);
		if ( arguments[1] && arguments[2] )
			x = x.replace(arguments[1],arguments[2]);
		for (i=0;i<Language.syntax.length;i++) 
			x = x.replace(Language.syntax[i].input,Language.syntax[i].output);
		editor.innerHTML = this.actions.history[this.actions.next()] = (flag=='scroll') ? x : o.replace(z,x);
		if (flag!='init') this.findString();
	},
	_prepareCodeForSyntax : function(flag)
	{
		var code = editor.innerHTML;
		if (flag == 'paste')
		{ // fix pasted text
			code = code.replace(/<BR>/g,'\r\n'); 
			code = code.replace(/\u2008/g,'\t');
		}
		code = code.replace(/<P>/g,'\n');
		code = code.replace(/<\/P>/g,'\r');
		code = code.replace(/<.*?>/g,'');
		code = code.replace(/&nbsp;/g,'');
		code = '<PRE><P>' + code + '</P></PRE>';
		code = code.replace(/\t/g,'<KBD>\t</KBD>');
		code = code.replace(/\n\r/g,'<P></P>');
		code = code.replace(/\n/g,'<P>');
		code = code.replace(/\r/g,'<\/P>');
		code = code.replace(/<P>(<P>)+/,'<P>');
		code = code.replace(/<\/P>(<\/P>)+/,'</P>');
		code = code.replace(/<P><\/P>/g,'<P><BR/></P>');
		return code;
	},
	snippets : function(evt)
	{
		var snippets = Language.snippets;
		var trigger = this.getLastWord();
		for (var i=0; i<snippets.length; i++)
		{
			if (snippets[i].input == trigger)
			{
				var content = snippets[i].output.replace(/</g,'&lt;');
				content = content.replace(/>/g,'&gt;');
				if (content.indexOf('$0') < 0)
					content += cc;
				else
					content = content.replace(/\$0/,cc);
				content = content.replace(/\n/g,'</P><P>');
				var pattern = new RegExp(trigger+cc,"gi");
				this.syntaxHighlight('snippets',pattern,content);
			}
		}
	},
	readOnly : function()
	{
		editor.contentEditable = (arguments[0]) ? 'false' : 'true';
	},
	complete : function(trigger)
	{
		var complete = Language.complete;
		for (var i=0; i<complete.length; i++)
		{
			if (complete[i].input == trigger)
			{
				var pattern = new RegExp('\\'+trigger+cc);
				var content = complete[i].output.replace(/\$0/g,cc);
				setTimeout(function () { CodePress.syntaxHighlight('complete',pattern,content)},0); // wait for char to appear on screen
			}
		}
	},
	getCompleteChars : function()
	{
		var cChars = '';
		for (var i=0;i<Language.complete.length;i++)
			cChars += '|'+Language.complete[i].input;
		return cChars+'|';
	},
	shortcuts : function()
	{
		var cCode = arguments[0];
		if (cCode==13)
			cCode = '[enter]';
		else if (cCode==32)
			cCode = '[space]';
		else
			cCode = '['+String.fromCharCode(keyCode).toLowerCase()+']';
		for (var i=0;i<Language.shortcuts.length;i++)
			if (Language.shortcuts[i].input == cCode)
				this.insertCode(Language.shortcuts[i].output,false);
	},
	getLastWord : function()
	{
		var rangeAndCaret = CodePress.getRangeAndCaret();
		words = rangeAndCaret[0].substring(rangeAndCaret[1]-40,rangeAndCaret[1]);
		words = words.replace(/[\s\n\r\);\W]/g,'\n').split('\n');
		return words[words.length-1].replace(/[\W]/gi,'').toLowerCase();
	}, 
	getRangeAndCaret : function()
	{
		var range = document.selection.createRange();
		var caret = Math.abs(range.moveStart('character', -1000000)+1);
		range = this.getCode();
		range = range.replace(/\n\r/gi,'  ');
		range = range.replace(/\n/gi,'');
		return [range.toString(),caret];
	},
	getSelectedText : function()
	{
		return document.selection.createRange().text;
	},
	getRange : function()
	{
		return document.selection.createRange();
	},
	insertCode : function(code,replaceCursorBefore)
	{
		var repdeb = '';
		var repfin = '';
		if (replaceCursorBefore)
			repfin = code;
		else
			repdeb = code;
		if (typeof document.selection != 'undefined')
		{
			var range = document.selection.createRange();
			range.text = repdeb + repfin;
			range = document.selection.createRange();
			range.move('character', -repfin.length);
			range.select();
		}
	},
	getCode : function()
	{
		return CodePress._convertCode(editor.innerHTML);
	},
	_convertCode : function(code)
	{
		code = code.replace(/<br>/g,'\n');
		code = code.replace(/<\/p>/gi,'\r');
		code = code.replace(/<p>/i,''); // IE first line fix
		code = code.replace(/<p>/gi,'\n');
		code = code.replace(/&nbsp;/gi,'');
		code = code.replace(/\u2009/g,'');
		code = code.replace(/<.*?>/g,'');
		code = code.replace(/&lt;/g,'<');
		code = code.replace(/&gt;/g,'>');
		code = code.replace(/&amp;/gi,'&');
		return code;
	},
	setCode : function()
	{
		var code = arguments[0];
		code = code.replace(/\u2009/gi,'');
		code = code.replace(/&/gi,'&amp;');
		code = code.replace(/</g,'&lt;');
		code = code.replace(/>/g,'&gt;');
		editor.innerHTML = '<pre>'+code+'</pre>';
	},
	actions : {
		pos : -1, // actual history position
		history : [], // history vector
		undo : function() {
			if (editor.innerHTML.indexOf(cc) == -1)
			{
				document.selection.createRange().text = cc;
				this.history[this.pos] = editor.innerHTML;
			}
			this.pos--;
			if (typeof(this.history[this.pos]) == 'undefined') this.pos++;
			editor.innerHTML = this.history[this.pos];
			CodePress.findString();
		},
		redo : function()
		{
			this.pos++;
			if (typeof(this.history[this.pos]) == 'undefined') this.pos--;
			editor.innerHTML = this.history[this.pos];
			CodePress.findString();
		},
		next : function()
		{
			if (this.pos > 20) this.history[this.pos-21] = undefined;
			return ++this.pos;
		}
	}
}

Language={};
window.attachEvent('onload', function() { CodePress.initialize('new');});
